﻿using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.OnlineCustomization.Entities;
using Kivii.OnlineCustomization.Requests;
using Kivii.Text;
using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Net.Sockets;
using System.Security.Cryptography;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Kivii.OnlineCustomization.Extensions
{
    public static class OrderExtension
    {
        /// <summary>
        /// 拉取订单
        /// </summary>
        /// <param name="baseUrl"></param>
        /// <param name="apptoken"></param>
        /// <returns></returns>
        public static RestfulQueryResponse<OnlineOrder> PullOrders(this JsonHttpClient _client, string apptoken, string appsecret, DateTime? beginTime = null, DateTime? endTime = null, int length = 200)
        {
            (_client == null).ThrowIfTrue("PullOrders:请传入_client");
            //(apptoken.IsNullOrEmpty()).ThrowIfTrue("PullOrders:请传入apptoken");
            //(appsecret.IsNullOrEmpty()).ThrowIfTrue("PullOrders:请传入appsecret");
            //var client = new JsonServiceClient(baseUrl);

            //var timeStamp = GetTimeStamp();
            //var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            //var signature = GetSign(appSign);
            var getWtsListRequest = new OnlineOrderGetWtsList();
            getWtsListRequest.Take = length;
            //getWtsListRequest.status = 0;
            if (beginTime != null && beginTime.Value != DateTime.MinValue) getWtsListRequest.BeginTime = beginTime;
            if (endTime != null && endTime.Value != DateTime.MinValue) getWtsListRequest.EndTime = endTime;

            var response = _client.Get(getWtsListRequest);
            return response;
        }

        public static RestfulUpdateResponse<OnlineOrder> RejectOrder(this JsonHttpClient _client, string orderNo, string apptoken = null, string appsecret = null)
        {
            (_client == null).ThrowIfTrue("RejectOrder:请传入_client");
            //if (apptoken.IsNullOrEmpty()) apptoken = Configs._token;//.ThrowIfTrue("RejectOrder:请传入apptoken");
            //if (appsecret.IsNullOrEmpty()) appsecret = Configs._secret;//.ThrowIfTrue("RejectOrder:请传入appsecret");
            (orderNo.IsNullOrEmpty()).ThrowIfTrue("RejectOrder:请传入orderNo");
            //var client = new JsonServiceClient(baseUrl);

            //var timeStamp = GetTimeStamp();
            //var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            //var signature = GetSign(appSign);
            //var paramsString = $"?appToken={apptoken}&signature={signature}&timestamp={timeStamp}";
            //var getWtsListRequest = new Dictionary<string, string>();
            //getWtsListRequest.Add("ORDER_NO", orderNo);
            var paramsString = $"?OrderId={orderNo}";
            var request = new OnlineOrderReject();
            request.OrderId = orderNo;
            try
            {
                var response = _client.Post(request);
                return response;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// 回传报告PDF
        /// </summary>
        /// <param name="_client"></param>
        /// <param name="apptoken"></param>
        /// <param name="appsecret"></param>
        /// <param name="orderNo"></param>
        /// <param name="orderPdfs"></param>
        /// <returns></returns>
        public static RestfulUpdateResponse<OnlineOrder> UploadOrderPdf(this JsonHttpClient _client, string apptoken, string appsecret, string orderNo, List<string> pdfUrls)
        {
            (_client == null).ThrowIfTrue("UploadOrderPdf:请传入_client");
            //(apptoken.IsNullOrEmpty()).ThrowIfTrue("UploadOrderPdf:请传入apptoken");
            //(appsecret.IsNullOrEmpty()).ThrowIfTrue("UploadOrderPdf:请传入appsecret");
            orderNo.ThrowIfNullOrEmpty("请传入订单号");
            pdfUrls.ThrowIfNullOrEmpty("UploadOrderPdf:请传入回传内容");
            var request = new OnlineOrderUploadResults();
            request.ReportPaths = new List<string>();
            request.ReportPaths.AddRange(pdfUrls);
            request.OrderId = orderNo;
            var response = _client.Post(request);
            return response;
        }

        /// <summary>
        /// 创建第三方订单数据
        /// </summary>
        /// <param name="orders"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static List<ThirdReportOrder> AcceptThirdContractOrder(this List<OnlineOrder> orders, Guid? ownerKvid = null, string ownerName = null, IDbConnection conn = null)
        {
            (orders.IsNullOrEmpty()).ThrowIfTrue("AcceptThirdContractOrder:请传入orders");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            }

            var rtns = new List<ThirdReportOrder>();

            //找出最小创建日期的订单
            var queryThirdReportOrders = conn.From<ThirdReportOrder>();
            queryThirdReportOrders.Where(o => o.ThirdType == typeof(OnlineOrder).FullName);
            queryThirdReportOrders.Select(o => o.SerialNumber);
            var thirdReportOrders = conn.Column<string>(queryThirdReportOrders);

            try
            {
                foreach (var item in orders)
                {
                    if (thirdReportOrders.Exists(o => o == item.OrderId)) continue;

                    var third = new ThirdReportOrder();
                    third.SerialNumber = item.OrderId;
                    third.Name = item.BuyerName;
                    third.ThirdName = item.BuyerCompony;
                    third.ThirdType = typeof(OnlineOrder).FullName;
                    third.IsAccepted = false;
                    third.IsUploaded = false;
                    third.OperateTime = DateTime.Now;

                    third.ManufacturerName = item.BuyerCompony;
                    third.ManufacturerContactName = item.BuyerName;
                    third.Metadata = new Dictionary<string, string>();
                    var pros = item.GetType().GetProperties();
                    foreach (var propertyInfo in pros)
                    {
                        if (propertyInfo.Name == "Metadata")
                        {
                            var items = item.Metadata;
                            if (items.IsNullOrEmpty()) items = new Dictionary<string, string>();
                            third.Metadata.Add(propertyInfo.Name, JsonSerializer.SerializeToString(items));
                            continue;
                        }

                        if (propertyInfo.Name == "Items")
                        {
                            var items = item.Items;
                            if (items.IsNullOrEmpty()) items = new List<OnlineOrderItem>();
                            third.Metadata.Add(propertyInfo.Name, JsonSerializer.SerializeToString(items));
                            continue;
                        }
                        var value = propertyInfo.GetValue(item);
                        third.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                    }
                    conn.Insert(third);
                    //trans?.Commit();
                    third.RemoveAllOnlyProperties();
                    rtns.Add(third);
                }
            }
            catch (Exception ex)
            {
                //trans?.Rollback();
                throw ex;
            }

            return rtns;
        }

        public static void LinkReports(List<ThirdReportOrder> thirds = null, JsonServiceClient _client = null)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            if (thirds.IsNullOrEmpty())
            {
                thirds = conn.Select<ThirdReportOrder>(o => /*o.IsAccepted == false &*/ o.IsOffseted == false & o.IsUploaded == false & o.ThirdType == typeof(OnlineOrder).FullName);
            }
            if (thirds.IsNullOrEmpty()) return;
            var allThirdDetails = conn.Select<ThirdReportOrderDetail>(o => Sql.In(o.ThirdOrderKvid, thirds.ConvertAll(p => p.Kvid)));
            var queryReports = conn.From<Report>();
            queryReports.Where(o => Sql.In(o.BizId, thirds.ConvertAll(p => p.SerialNumber)) & o.Status >= (int)ReportStatusType.ReportIssue).OrderBy(o => o.ReportId);
            var allReports = conn.Select(queryReports);
            if (allReports.IsNullOrEmpty()) return;

            var client = new JsonHttpClient(Configs._defaultUrl);

            foreach (var third in thirds)
            {
                var reports = allReports.Where(o => o.BizId == third.SerialNumber).ToList();
                if (reports.IsNullOrEmpty()) continue;
                //if (reports.Exists(o => o.TestJudgement == "不合格"))
                //{
                //    third.ReportIds = "存在不合格报告请手动关联!";
                //    third.AddOnlyProperties(o => o.ReportIds);
                //    conn.UpdateOnly(third);
                //    continue;
                //}
                if (!allThirdDetails.IsNullOrEmpty())
                {
                    var thirdDetails = allThirdDetails.Where(o => o.ThirdOrderKvid == third.Kvid).ToList();
                    if (!thirdDetails.IsNullOrEmpty())
                    {
                        foreach (var node in thirdDetails)
                        {
                            node.Status = -1;
                            node.AddOnlyProperties(o => o.Status);
                            conn.UpdateOnly(node);
                        }
                    }
                }
                third.ReportIds = string.Empty;
                var reportIds = new List<string>();
                foreach (var report in reports)
                {
                    report.BizId = third.SerialNumber;
                    report.AddOnlyProperties(o => o.BizId);
                    report.BizKvid = third.Kvid;
                    report.AddOnlyProperties(o => o.BizKvid);
                    report.BizType = typeof(ThirdReportOrder).FullName;
                    report.AddOnlyProperties(o => o.BizType);
                    conn.UpdateOnly(report);

                    var thirdDetail = new ThirdReportOrderDetail();
                    thirdDetail.ThirdOrderKvid = third.Kvid;
                    thirdDetail.ReportId = report.ReportId;
                    thirdDetail.ReportKvid = report.Kvid;
                    thirdDetail.ThirdType = third.ThirdType;
                    thirdDetail.SerialNumber = third.SerialNumber;
                    thirdDetail.Name = third.Name;
                    thirdDetail.ThirdName = third.ThirdName;
                    thirdDetail.IsUploaded = false;
                    thirdDetail.OperateTime = DateTime.Now;
                    thirdDetail.Metadata = new Dictionary<string, string>();
                    thirdDetail.Metadata = report.Metadata;
                    foreach (var propertyInfo in report.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "Metadata") continue;
                        try
                        {
                            var value = propertyInfo.GetValue(report);
                            thirdDetail.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                        }
                        catch { }

                    }
                    conn.Insert(thirdDetail);
                    reportIds.Add(report.ReportId);
                }

                if (third.Metadata.IsNullOrEmpty()) third.Metadata = new Dictionary<string, string>();
                third.Metadata["Amount"] = reports.Sum(o => o.Amount + o.AmountTest).ToString();
                third.AddOnlyProperties(o => o.Metadata);
                third.ReportIds = string.Join("、", reportIds);
                third.AddOnlyProperties(o => o.ReportIds);
                third.IsAccepted = true;
                third.AddOnlyProperties(o => o.IsAccepted);
                conn.UpdateOnly(third);
                if (_client != null) _client.ConfirmAccept(third);
            }
        }

        public static string GetTimeStamp()
        {
            System.DateTime startTime = TimeZone.CurrentTimeZone.ToLocalTime(new System.DateTime(1970, 1, 1)); // 当地时区
            var timeStamp = ((int)(DateTime.Now - startTime).TotalSeconds).ToString(); // 相差秒数
            return timeStamp;
            //return string.Format("{0:yyyyMMddHHmmss}", DateTime.Now);//$"{DateTime.Now:yyyyMMddHHmmss}";
        }

        /// <summary>
        /// apptoken=12345&appsecret=56789&timestamp=1632127400
        /// </summary>
        /// <param name="accessKey"></param>
        /// <returns></returns>
        public static string GetSign(string accessKey)
        {
            if (accessKey.IsNullOrEmpty()) accessKey = string.Empty;
            MD5 md5 = new MD5CryptoServiceProvider();
            byte[] targetData = md5.ComputeHash(System.Text.Encoding.UTF8.GetBytes(accessKey));
            StringBuilder sign = new StringBuilder("");
            foreach (byte b in targetData)
            {
                sign.AppendFormat("{0:x2}", b);
            }
            return sign.ToString().ToUpper();
        }

        /// <summary>
        /// 字典类型转化为对象
        /// </summary>
        /// <param name="dic"></param>
        /// <returns></returns>
        public static OnlineOrder DicToObject(this Dictionary<string, string> dic)
        {
            var md = new OnlineOrder();
            CultureInfo cultureInfo = Thread.CurrentThread.CurrentCulture;
            TextInfo textInfo = cultureInfo.TextInfo;
            foreach (var d in dic)
            {
                var filed = textInfo.ToTitleCase(d.Key);
                try
                {
                    var value = d.Value;
                    var property = md.GetType().GetProperty(filed);
                    if (property == null) continue;
                    if (!property.PropertyType.IsGenericType)
                    {
                        switch (property.Name)
                        {
                            case "Metadata":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<Dictionary<string, string>>(value), null);
                                break;
                            case "Items":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<OnlineOrderItem>>(value), null);
                                break;
                            default:
                                if (string.IsNullOrEmpty(value))
                                {
                                    property.SetValue(md, null, null);
                                }
                                else
                                {
                                    // 特殊处理Guid类型
                                    if (property.PropertyType == typeof(Guid) || property.PropertyType == typeof(Guid?))
                                    {
                                        if (property.PropertyType == typeof(Guid?))
                                        {
                                            // 可空Guid
                                            Guid guidValue;
                                            if (Guid.TryParse(value, out guidValue))
                                            {
                                                property.SetValue(md, guidValue, null);
                                            }
                                            else
                                            {
                                                property.SetValue(md, null, null);
                                            }
                                        }
                                        else
                                        {
                                            // 非空Guid
                                            Guid guidValue;
                                            if (Guid.TryParse(value, out guidValue))
                                            {
                                                property.SetValue(md, guidValue, null);
                                            }
                                            else
                                            {
                                                property.SetValue(md, Guid.Empty, null);
                                            }
                                        }
                                    }
                                    else
                                    {
                                        // 其他类型使用Convert.ChangeType
                                        property.SetValue(md, Convert.ChangeType(value, property.PropertyType), null);
                                    }
                                }
                                break;
                        }
                    }
                    else
                    {
                        Type genericTypeDefinition = property.PropertyType.GetGenericTypeDefinition();
                        switch (property.Name)
                        {
                            case "Metadata":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<Dictionary<string, string>>(value), null);
                                break;
                            case "Items":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<OnlineOrderItem>>(value), null);
                                break;
                            default:
                                //泛型Nullable<>
                                if (genericTypeDefinition == typeof(Nullable<>))
                                {
                                    if (string.IsNullOrEmpty(value))
                                    {
                                        property.SetValue(md, null, null);
                                    }
                                    else
                                    {
                                        // 特殊处理Guid类型
                                        if (property.PropertyType == typeof(Guid) || property.PropertyType == typeof(Guid?))
                                        {
                                            if (property.PropertyType == typeof(Guid?))
                                            {
                                                // 可空Guid
                                                Guid guidValue;
                                                if (Guid.TryParse(value, out guidValue))
                                                {
                                                    property.SetValue(md, guidValue, null);
                                                }
                                                else
                                                {
                                                    property.SetValue(md, null, null);
                                                }
                                            }
                                            else
                                            {
                                                // 非空Guid
                                                Guid guidValue;
                                                if (Guid.TryParse(value, out guidValue))
                                                {
                                                    property.SetValue(md, guidValue, null);
                                                }
                                                else
                                                {
                                                    property.SetValue(md, Guid.Empty, null);
                                                }
                                            }
                                        }
                                        else
                                        {
                                            // 其他类型使用Convert.ChangeType
                                            property.SetValue(md, Convert.ChangeType(value, Nullable.GetUnderlyingType(property.PropertyType)), null);
                                        }
                                    }
                                }
                                break;
                        }
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
            return md;
        }

        internal static bool ConfirmAccept(this JsonServiceClient _client, ThirdReportOrder thirdReportOrder)
        {
            (_client == null).ThrowIfTrue("ConfirmAccept:请传入_client");
            var acceptRequest = new OnlineOrderAccept();
            acceptRequest.OrderId = thirdReportOrder.SerialNumber;
            var response = _client.Post<OnlineOrderResponse>(acceptRequest);
            if (!response.Success) return false;
            return true;
        }
    }
}
